import os
from math import sqrt, pi, sin, cos
from shallow_water_h import *
from Numeric import allclose, array, zeros, ones, Float, take, sqrt
from config import g, epsilon

def newLinePlot(title='Simple Plot'):
    import Gnuplot
    g = Gnuplot.Gnuplot(persist=0)
    g.title(title)
    g('set data style linespoints') 
    g.xlabel('x')
    g.ylabel('y')
    return g

def linePlot(g,x1,y1,x2,y2,x3,y3):
    import Gnuplot
    plot1 = Gnuplot.PlotItems.Data(x1.flat,y1.flat,with="lines 2")
    plot2 = Gnuplot.PlotItems.Data(x2.flat,y2.flat,with="lines 3")
    plot3 = Gnuplot.PlotItems.Data(x3.flat,y3.flat,with="linespoints 1")
    g.plot(plot1,plot2,plot3)


def analytic_cannal(C,t):
    N = len(C)
    u = zeros(N,Float)    ## water velocity
    h = zeros(N,Float)    ## water depth
    x = C
    g = 9.81


    ## Define Basin Bathymetry
    z_b = zeros(N,Float) ## elevation of basin
    z = zeros(N,Float)   ## elevation of water surface
    z_infty = 10.0       ## max equilibrium water depth at lowest point.
    L_x = 2500.0         ## width of channel

    A0 = 0.5*L_x                  ## determines amplitudes of oscillations
    omega = sqrt(2*g*z_infty)/L_x ## angular frequency of osccilation

    for i in range(N):
        z_b[i] = z_infty*(x[i]**2/L_x**2) ## or A0*cos(omega*t)\pmL_x
        u[i] = -A0*omega*sin(omega*t)
        z[i] = z_infty+2*A0*z_infty/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t))

    h = z-z_b
    return u,h,z,z_b


#plot2 = newLinePlot("Momentum")  

L_x = 2500.0     # Length of channel (m)
N = 100          # Number of compuational cells
cell_len = 4*L_x/N   # Origin = 0.0

points = zeros(N+1,Float)
for i in range(N+1):
        points[i] = -2*L_x +i*cell_len

domain = Domain(points)

domain.default_order = 2
domain.default_time_order = 2
domain.cfl = 1.0
domain.beta = 1.0
domain.flux_first = False



def stage(x):

    z_infty = 10.0       ## max equilibrium water depth at lowest point.
    L_x = 2500.0         ## width of channel
    A0 = 0.5*L_x                  ## determines amplitudes of oscillations
    omega = sqrt(2*g*z_infty)/L_x ## angular frequency of osccilation
    t=0.0
    y = zeros(len(x),Float)
    for i in range(len(x)):
        y[i] = z_infty+2*A0*z_infty/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t))
    return y

def elevation(x):
    N = len(x)
    z_infty = 10.0
    z = zeros(N,Float)
    L_x = 2500.0
    A0 = 0.5*L_x
    omega = sqrt(2*g*z_infty)/L_x
    for i in range(N):
        z[i] = z_infty*(x[i]**2/L_x**2)
    return z

def height(x):
    z_infty = 10.0       ## max equilibrium water depth at lowest point.
    L_x = 2500.0         ## width of channel
    A0 = 0.5*L_x                  ## determines amplitudes of oscillations
    omega = sqrt(2*g*z_infty)/L_x ## angular frequency of osccilation
    t=0.0
    y = zeros(len(x),Float)
    for i in range(len(x)):
        y[i] = max(z_infty+2*A0*z_infty/L_x*cos(omega*t)*(x[i]/L_x-0.5*A0/(L_x)*cos(omega*t))-z_infty*(x[i]**2/L_x**2),0.0)
        #y[i] = 12.0-z_infty*(x[i]**2/L_x**2)
    return y

    
#domain.set_quantity('stage', stage)
domain.set_quantity('elevation',elevation)
domain.set_quantity('height',height)
#print "Centroid Heights"
#print domain.quantities['height'].centroid_values
#print "Vertex Heights"
#print domain.quantities['height'].vertex_values
#print "Centroids Stage"
#print domain.quantities['height'].centroid_values+domain.quantities['elevation'].centroid_values
#raw_input()
domain.set_boundary({'exterior': Reflective_boundary(domain)})
 
import time
t0 = time.time()
#yieldstep = 50.0
yieldstep = 1122.0
#yieldstep = 1.0
finaltime = 1122.0

plot1 = newLinePlot("Stage")
plot2 = newLinePlot("Xmomentum")
C = domain.centroids
X = domain.vertices
HeightQ = domain.quantities['height'].vertex_values
ElevationQ = domain.quantities['elevation'].vertex_values
XmomQ = domain.quantities['xmomentum'].vertex_values

#StageQ = domain.quantities['stage'].centroid_values       
StageQ = ElevationQ+HeightQ
u,h,z,z_b = analytic_cannal(X.flat,0.0)
linePlot(plot1,X,z,X,z_b,X,StageQ)
raw_input()
print "integral height %f"%domain.quantities['height'].get_integral()
print "integral xmom %f"%domain.quantities['xmomentum'].get_integral()
C = domain.centroids
import time
t0 = time.time()
for t in domain.evolve(yieldstep = yieldstep, finaltime = finaltime):
    domain.write_time()
    print "integral height %f"%domain.quantities['height'].get_integral()
    print "integral xmom %f"%domain.quantities['xmomentum'].get_integral()
    u,h,z,z_b = analytic_cannal(X.flat,t)
    StageQ = HeightQ+ElevationQ
    #print "yield",HeightQ
    linePlot(plot1,X,z,X,z_b,X,StageQ)
    linePlot(plot2,X,u*h,X,u*h,X,XmomQ)
    u,hc,z,z_b = analytic_cannal(C,t)
    HeightQC = domain.quantities['height'].centroid_values
    #print HeightQC
    for k in range(N):
        if hc[k] < 0.0:
            hc[k] = 0.0
    error = 1.0/(N)*sum(abs(hc-HeightQC))
    print 'Error measured at centroids', error
   # raw_input()
    
print 'That took %.2f seconds' %(time.time()-t0)
